@@ -1,7 +1,7 @@
 // drumkv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -50,7 +50,7 @@ class drumkv1widget : public QWidget
 public:
 
 	// Constructor
-	drumkv1widget(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
+	drumkv1widget(QWidget *pParent = nullptr);
 
 	// Destructor.
 	virtual ~drumkv1widget();
@@ -139,6 +139,9 @@ protected slots:
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
+	// Panic: all-notes/sound-off (reset).
+	void panic();
+	
 	// Offset point changed.
 	void offsetStartChanged();
 	void offsetEndChanged();
